function Out-NtapDocument {
    <#
        .SYNOPSIS
            Writes output documentation using the output from the Format-Ntap* functions.

        .DESCRIPTION
            Outputs documentation files containing parsed configuration information gathered by the Get-Ntap* functions
            and formatted by the Format-Ntap* functions. Also accepts pipeline input from the Format-Ntap* functions.

        .EXAMPLE
            PS> Out-NtapDocument -InputObject 'D:\Site.xml' -ExcelFile 'D:\Output\Site.xlsx'

            Outputs the data contained in D:\site.xml and produces an Excel document.

        .EXAMPLE
            PS> Out-NtapDocument -InputObject 'D:\Site.xml' -WordTemplate 'D:\template.dotx' -WordFile 'D:\Output\Site.docx' -CustomerName 'NetAppDocs Customer' -CustomerLocation 'Seattle, WA'

            Outputs the data contained in D:\site.xml and produces a Word document based on a custom template. To create a
            custom template, copy the template found in the NetAppDocs module 'Templates' folder. The most common uses for
            creating a custom template are to modify the 'Overview' section and add an 'Appendix' section for customer
            specific static information.

        .EXAMPLE
            PS> Out-NtapDocument -InputObject 'D:\Site.xml' -WordFile 'D:\Output\Site.docx' -VisioFile 'D:\Output\Site.vsd' -CustomerName "NetAppDocs Customer" -CustomerLocation "Seattle, WA" -SaveWordDocAsPdf

            Outputs the data contained in D:\site.xml and produces a Word document.

        .EXAMPLE
            PS> Get-ChildItem -Path 'D:\Data' -Filter "*-parsed.xml" | Out-NtapDocument -WordFile 'D:\Output\Site.docx' -ExcelFile 'D:\Output\Site.xlsx' -CustomerName "NetAppDocs Customer" -CustomerLocation "Seattle, WA"

            This will gather all the parsed XML files previously output from this function (using only the -XmlFile
            parameter). A Word document and an Excel document will be created and the cover page of the Word document will
            be updated with the customer name and location.

        .LINK
            https://community.netapp.com/t5/NetAppDocs/bd-p/netappdocs

        .LINK
            Update-NtapDocument

        .INPUTS
            System.Data.DataSet

        .INPUTS
            System.IO.FileInfo

        .INPUTS
            System.String

        .OUTPUTS
            None

        .NOTES
            AUTHOR : Jason Cole
    #>

    [CmdletBinding()]
    [OutputType([System.Void])]
    param (
        # A DataSet or an XML file created by the Format-Ntap* functions (might be *.xml.gz).
        [Parameter(Mandatory, Position = 0, ValueFromPipeline)]
        [ValidateScript({
                if ($_ -isnot [System.Data.DataSet]) {
                    if ($_ -is [System.IO.FileInfo] -or $_ -is [System.String]) {
                        if ([System.IO.Path]::GetExtension($_) -ne '.xml' -and [System.IO.Path]::GetExtension($_) -ne '.gz') {
                            throw (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the InputObject parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $_))
                        }
                    }
                    else {
                        throw (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid input object specified.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $_))
                    }
                }

                $true
            })]
        [Alias('InputFile', 'InputData')]
        [System.Object[]]$InputObject,

        # The path to the parsed output XML file.
        [Parameter()]
        [string]$XmlFile,

        # The path to the output Excel document.
        [Parameter()]
        [string]$ExcelFile,

        # The path to the output Excel template.
        [Parameter()]
        [string]$ExcelTemplate,

        # The path to the output Word document.
        [Parameter()]
        [string]$WordFile,

        # The path to the output Word template.
        [Parameter()]
        [string]$WordTemplate,

        # The output document paper size. Currently, only Letter, A4 and P4 are supported. (Default: Letter)
        [Parameter()]
        [ValidateSet('Letter', 'A4', 'P4')]
        [string]$PaperSize,

        # The path to an existing Visio file to copy drawings from.
        [Parameter()]
        [string]$VisioFile,

        # Saves the Word document to a PDF file.
        [Parameter()]
        [switch]$SaveWordDocAsPdf,

        # The customer name to add to the documentation.
        [Parameter()]
        [string]$CustomerName,

        # The customer location to add to the documentation.
        [Parameter()]
        [string]$CustomerLocation,

        # The project name to add to the documentation.
        [Parameter()]
        [string]$ProjectName,

        # The author name to add to the documentation.
        [Parameter()]
        [string]$AuthorName,

        # Specifies the locale for the default templates.
        [Parameter()]
        [ValidateSet('en-US')]
        [string]$Locale = 'en-US'
    )

    begin {
        if ([string]::IsNullOrWhiteSpace($XmlFile) -and [string]::IsNullOrWhiteSpace($WordFile) -and [string]::IsNullOrWhiteSpace($ExcelFile)) {
            Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ('No valid output files passed to function.', 'InvalidArgument', [System.Management.Automation.ErrorCategory]::InvalidArgument, 'Out-NtapDocument'))

            continue
        }

        if (-not [string]::IsNullOrWhiteSpace($XmlFile)) {
            if ([System.IO.Path]::GetExtension($XmlFile) -ne '.xml') {
                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the XmlFile parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $XmlFile))

                continue
            }

            try {
                $xmlFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath($XmlFile)
            }
            catch {
                $Global:Error.RemoveAt(0)

                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file path specified for the XmlFile parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $XmlFile))

                continue
            }

            if ($xmlFileInfo.Exists) {
                [void](Rename-Item -Path $xmlFileInfo.FullName -NewName ([string]::Format("$( $xmlFileInfo.BaseName )-Copy_{0:yyyyMMdd-HHmm}{1}", $xmlFileInfo.CreationTime, $xmlFileInfo.Extension)) -ErrorAction SilentlyContinue)
            }
            else {
                if (-not $xmlFileInfo.Directory.Exists) {
                    try {
                        [void](New-Item -Path $xmlFileInfo.DirectoryName -ItemType Directory -ErrorAction Stop)
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to create destination folder: $( $xmlFileInfo.DirectoryName )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $xmlFileInfo))

                        continue
                    }
                }
            }

            $xsdFileInfo = [System.IO.FileInfo]([System.IO.Path]::ChangeExtension($xmlFileInfo.FullName, '.xsd'))

            if (Test-Path -Path $xsdFileInfo -PathType Leaf) {
                [void](Rename-Item -Path $xsdFileInfo -NewName ([string]::Format("$( $xsdFileInfo.BaseName )-Copy_{0:yyyyMMdd-HHmm}{1}", $xsdFileInfo.CreationTime, $xsdFileInfo.Extension)) -ErrorAction SilentlyContinue)
            }
        }

        if (-not [string]::IsNullOrWhiteSpace($ExcelFile)) {
            if ([System.IO.Path]::GetExtension($ExcelFile) -ne '.xlsx') {
                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the ExcelFile parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $ExcelFile))

                continue
            }

            try {
                $excelFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath($ExcelFile)
            }
            catch {
                $Global:Error.RemoveAt(0)

                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file path specified for the ExcelFile parameter.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $ExcelFile))

                continue
            }

            if ($excelFileInfo.Exists) {
                [void](Rename-Item -Path $excelFileInfo -NewName ([string]::Format("$( $excelFileInfo.BaseName )-Copy_{0:yyyyMMdd-HHmm}{1}", $excelFileInfo.CreationTime, $excelFileInfo.Extension)) -ErrorAction SilentlyContinue)
            }
            else {
                if (-not $excelFileInfo.Directory.Exists) {
                    try {
                        [void](New-Item -Path $excelFileInfo.DirectoryName -ItemType Directory -ErrorAction Stop)
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to create destination folder: $( $excelFileInfo.DirectoryName )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $excelFileInfo.DirectoryName))

                        continue
                    }
                }
            }
        }

        if (-not [string]::IsNullOrWhiteSpace($WordFile)) {
            if ([System.IO.Path]::GetExtension($WordFile) -ne '.docx') {
                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the WordFile parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $WordFile))

                continue
            }

            try {
                $wordFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath($WordFile)
            }
            catch {
                $Global:Error.RemoveAt(0)

                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file path specified for the WordFile parameter.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $WordFile))

                continue
            }

            if ($wordFileInfo.Exists) {
                [void](Rename-Item -Path $wordFileInfo -NewName ([string]::Format("$( $wordFileInfo.BaseName )-Copy_{0:yyyyMMdd-HHmm}{1}", $wordFileInfo.CreationTime, $wordFileInfo.Extension)) -ErrorAction SilentlyContinue)
            }
            else {
                if (-not $wordFileInfo.Directory.Exists) {
                    try {
                        [void](New-Item -Path $wordFileInfo.DirectoryName -ItemType Directory -ErrorAction Stop)
                    }
                    catch {
                        $Global:Error.RemoveAt(0)

                        Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to create destination folder: $( $wordFileInfo.DirectoryName )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $wordFileInfo.DirectoryName))

                        continue
                    }
                }
            }
        }

        if (-not [string]::IsNullOrWhiteSpace($VisioFile)) {
            if ($PSVersionTable.PSVersion -ge [version]::Parse('7.0') -and -not $IsWindows) {
                Write-Warning -Message 'Copying Visio drawings to the Word document is not currently supported on non-Windows platforms. See the FAQ entry for more details.'
            }
            else {
                if ([System.IO.Path]::GetExtension($VisioFile) -notlike '.vsd*') {
                    Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the VisioFile parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $VisioFile))
                }

                try {
                    $visioFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetResolvedPSPathFromPSPath($VisioFile).ProviderPath
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    Write-Warning -Message "Specified VisioFile does not exist. Drawings will not be copied to the Word Document.`n         Note: NetAppDocs does not currently support generating Visio drawings. This functionality will be available in a future release."
                }
            }
        }

        if (-not [string]::IsNullOrWhiteSpace($ExcelTemplate)) {
            if ([System.IO.Path]::GetExtension($ExcelTemplate) -ne '.xltx') {
                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the ExcelTemplate parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $ExcelTemplate))

                continue
            }

            try {
                $excelTemplateFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetResolvedPSPathFromPSPath($ExcelTemplate).ProviderPath
            }
            catch {
                $Global:Error.RemoveAt(0)

                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'ExcelTemplate specified does not exist.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $ExcelTemplate))

                continue
            }
        }

        if (-not [string]::IsNullOrWhiteSpace($WordTemplate)) {
            if ([System.IO.Path]::GetExtension($WordTemplate) -ne '.dotx') {
                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'Invalid file extension specified for the WordTemplate parameter.', 'ArgumentException', [System.Management.Automation.ErrorCategory]::InvalidArgument, $WordTemplate))

                continue
            }

            try {
                $wordTemplateFileInfo = [System.IO.FileInfo]$ExecutionContext.SessionState.Path.GetResolvedPSPathFromPSPath($WordTemplate).ProviderPath
            }
            catch {
                $Global:Error.RemoveAt(0)

                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ([System.ArgumentException]'WordTemplate specified does not exist.', $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidArgument, $WordTemplate))

                continue
            }
        }

        $dataSet = $null
    }
    process {
        Write-Verbose -Message "Initializing function: $( $PSCmdlet.MyInvocation.MyCommand.Name ) ($( $PSCmdlet.MyInvocation.MyCommand.ModuleName ) v$( (Get-NtapDocsVersion -AsVersion).ToString() ))"

        $functionPSCallStack = Get-PSCallStack

        foreach ($object in $InputObject) {
            if ($null -eq $object) {
                continue
            }

            try {
                $inputDataSet = Read-DataSet -InputObject $object
            }
            catch {
                $PSCmdlet.ThrowTerminatingError($_)
            }

            if (-not $dataSet) {
                $dataSet = $inputDataSet.Copy()
            }
            else {
                try {
                    if ($inputDataSet.DataSetName -ne $dataSet.DataSetName -or ($inputDataSet.ExtendedProperties['BaseTypeName'] -ne $dataSet.ExtendedProperties['BaseTypeName'])) {
                        throw 'All DataSets must be of the same system type.'
                    }

                    $primaryKeyColumnOrdinal = $inputDataSet.Tables['DocumentVersionInformation'].PrimaryKey[0].Ordinal
                    if ((Compare-Object -ReferenceObject $inputDataSet.Tables['DocumentVersionInformation'].get_item($primaryKeyColumnOrdinal) -DifferenceObject $dataSet.Tables['DocumentVersionInformation'].get_item($primaryKeyColumnOrdinal) -ExcludeDifferent -IncludeEqual | Measure-Object).Count -gt 0) {
                        #Should we save the XML files out if this happens?
                        throw 'Reason: Duplicate systems exist in the DataSets.'
                    }

                    $inputDataSet1Version = Get-NtapDocsDataSetSchemaVersion -DataSet $inputDataSet
                    $inputDataSet2Version = Get-NtapDocsDataSetSchemaVersion -DataSet $dataSet

                    if ($inputDataSet1Version -ne $inputDataSet2Version) {
                        throw 'The DataSets were generated using different versions of NetAppDocs.'
                    }

                    for ($t = 0; $t -lt $dataSet.Tables.Count; $t++) {
                        $mergedDataTable = $dataSet.Tables[$t]
                        $inputDataTable = $inputDataSet.Tables[$mergedDataTable.TableName]

                        if (($mergedDataTable.Rows.Count -gt 0 -and -not $mergedDataTable.PrimaryKey) -and $inputDataTable.PrimaryKey) {
                            Write-Verbose -Message "Removing PrimaryKey on input table: $( $mergedDataTable.TableName )"

                            $inputDataTable.PrimaryKey | ForEach-Object {
                                $_.AllowDBNull = $true
                            }

                            $inputDataTable.PrimaryKey = $null
                        }

                        if (($inputDataTable.Rows.Count -gt 0 -and -not $inputDataTable.PrimaryKey) -and $mergedDataTable.PrimaryKey) {
                            Write-Verbose -Message "Removing PrimaryKey on merged table: $( $mergedDataTable.TableName )"

                            $mergedDataTable.PrimaryKey | ForEach-Object {
                                $_.AllowDBNull = $true
                            }

                            $mergedDataTable.PrimaryKey = $null
                        }
                    }
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to merge DataSets. Reason: $( $_.Exception.Message )", $_.FullyQualifiedErrorId, $_.CategoryInfo.Category, $null)))
                }

                try {
                    Write-Verbose -Message 'Merging DataSets'

                    $dataSet.Merge($inputDataSet, $true, [System.Data.MissingSchemaAction]::AddWithKey)
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    $PSCmdlet.ThrowTerminatingError((New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Error merging DataSets. Reason: $( $_.Exception.Message )", $_.FullyQualifiedErrorId, [System.Management.Automation.ErrorCategory]::InvalidData, $null)))
                }
            }
        }
    }
    end {
        if (-not $InputObject) {
            return
        }

        if (-not $dataSet -or $dataSet.Tables.Count -eq 0) {
            Show-ErrorMessage -Message 'Empty DataSet found so document generation not possible. This indicates that an error has occurred during data collection or during formatting. See the FAQ entry for more information on determining the cause.'

            return
        }

        if (-not [string]::IsNullOrWhiteSpace($WordFile) -or -not [string]::IsNullOrWhiteSpace($ExcelFile)) {
            $missingParams = (Compare-Object -ReferenceObject 'CustomerName', 'CustomerLocation', 'ProjectName', 'AuthorName' -DifferenceObject @( $PSCmdlet.MyInvocation.BoundParameters.Keys | Select-Object ) | Where-Object { $_.SideIndicator -eq '<=' }).InputObject

            if ($missingParams.Count -gt 0) {
                Write-Warning -Message "The following document parameters were not specified: $( $missingParams -join ', '). See the relevant FAQ entry in the help pages for more information on how to manually update the cover pages of the output document(s)."
            }
        }

        $functionStartTime = [datetime]::Now

        $ntapDocsWriter = New-Object -TypeName NetAppDocs.Writer -ArgumentList $dataSet

        $ntapDocsWriter.XmlFile = $(
            if (-not [string]::IsNullOrWhiteSpace($XmlFile)) {
                $xmlFileInfo.FullName
            }
            else {
                if ($wordFileInfo) {
                    $ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath("$( $wordFileInfo.DirectoryName )/$( $wordFileInfo.BaseName )_tmpFmtd.xml")
                }
                elseif ($excelFileInfo) {
                    $ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath("$( $excelFileInfo.DirectoryName )/$( $excelFileInfo.BaseName )_tmpFmtd.xml")
                }
            }
        )

        if ($dataSet.HasErrors) {
            $ntapDocsWriter.XsdFile = [System.IO.Path]::ChangeExtension($ntapDocsWriter.XmlFile, '.xsd')
        }

        if (-not [string]::IsNullOrWhiteSpace($XmlFile)) {
            try {
                Write-Verbose -Message 'Generating XML document'

                $ntapDocsWriter.CreateXmlFile()
            }
            catch {
                $Global:Error.RemoveAt(0)

                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("XML file creation failed. Reason: $( $_.Exception.Message )", 'WriteError', [System.Management.Automation.ErrorCategory]::WriteError, $ntapDocsWriter.XmlFile))
            }
        }

        $ntapDocsWriter.PaperSize = $PaperSize
        $ntapDocsWriter.CustomerName = $CustomerName
        $ntapDocsWriter.CustomerLocation = $CustomerLocation
        $ntapDocsWriter.ProjectName = $ProjectName
        $ntapDocsWriter.AuthorName = $AuthorName

        if ($WordFileInfo) {
            Write-Verbose -Message 'Generating Word document'

            $defaultWordTemplate = Get-NtapWordTemplate -DataSetName $dataSet.DataSetName -BaseTypeName $dataSet.ExtendedProperties['BaseTypeName'] -Locale $Locale

            if (-not $defaultWordTemplate) {
                Write-Warning -Message 'Word document output not supported for this DataSet'
            }
            else {
                $ntapDocsWriter.WordFile = $wordFileInfo.FullName

                $ntapDocsWriter.WordTemplate = ($wordTemplateFileInfo.FullName, $defaultWordTemplate)[[byte](! $wordTemplateFileInfo)]

                if ($VisioFileInfo) {
                    $ntapDocsWriter.VisioFile = $visioFileInfo.FullName
                }

                try {
                    $ntapDocsWriter.CreateWordFile()

                    if ($SaveWordDocAsPdf.IsPresent) {
                        try {
                            Write-Verbose -Message 'Generating PDF document'

                            $ntapDocsWriter.CreatePdfFile()
                        }
                        catch {
                            $Global:Error.RemoveAt(0)

                            Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ("Unable to save Word document as a PDF file. Reason: $( $_.Exception.Message )", 'WriteError', [System.Management.Automation.ErrorCategory]::WriteError, $ntapDocsWriter.WordFile))
                        }
                    }
                }
                catch {
                    $Global:Error.RemoveAt(0)

                    $msg = New-Object -TypeName System.Text.StringBuilder

                    if ($_.Exception.Message -like '*Unable to add Visio drawings to Word document.*') {
                        [void]$msg.AppendLine('Unable to add Visio drawings to Word document.')
                        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
                        [void]$msg.AppendLine('The output Word document was generated but the Visio drawings will need to be manually added.')
                        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
                        [void]$msg.AppendLine()
                    }
                    else {
                        if ([string]::IsNullOrWhiteSpace($XmlFile)) {
                            $ntapDocsWriter.CreateXmlFile()
                        }

                        [void]$msg.AppendLine("Word document output creation failed. Reason: $( $_.Exception.Message ).")
                        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
                        [void]$msg.AppendLine("XML file may be used to complete document generation ($( $ntapDocsWriter.XmlFile )).")
                        [void]$msg.AppendLine("Example: C:\> Get-ChildItem -Path $( $ntapDocsWriter.XmlFile ) | Out-NtapDocument -WordFile <WordFile> -ExcelFile <ExcelFile> -CustomerName <CustomerName> -CustomerLocation <CustomerLocation>")
                        [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
                        [void]$msg.AppendLine()
                    }

                    Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ($msg.ToString(), 'WriteError', [System.Management.Automation.ErrorCategory]::WriteError, $ntapDocsWriter.WordFile))
                }
            }
        }

        if ($ExcelFileInfo) {
            Write-Verbose -Message 'Generating Excel document'

            $ntapDocsWriter.ExcelFile = $excelFileInfo.FullName

            $ntapDocsWriter.ExcelTemplate = $(
                if ($excelTemplateFileInfo) {
                    $excelTemplateFileInfo.FullName
                }
                else {
                    Get-NtapExcelTemplate -DataSetName $dataSet.DataSetName -BaseTypeName $dataSet.ExtendedProperties['BaseTypeName'] -Locale $Locale
                }
            )

            if ($ntapDocsWriter.ExcelTemplate -eq $ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath("$( ([System.IO.FileInfo]$PSCmdlet.MyInvocation.MyCommand.Module.Path).DirectoryName )/Templates/$Locale/NetAppDocs Excel Template.xltx")) {
                Write-Warning -Message 'Default Excel template will be used for Excel file generation. The document title and subject will need to be manually entered.'
            }

            try {
                $ntapDocsWriter.CreateExcelFile()
            }
            catch {
                if ([string]::IsNullOrWhiteSpace($XmlFile)) {
                    $ntapDocsWriter.CreateXmlFile()
                }

                $msg = New-Object -TypeName System.Text.StringBuilder

                [void]$msg.AppendLine("Excel document output creation failed. Reason: $( $_.Exception.Message ).")
                [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
                [void]$msg.AppendLine("XML file may be used to complete document generation ($( $ntapDocsWriter.XmlFile )).")
                [void]$msg.AppendLine("Example: C:\> Get-ChildItem -Path $( $ntapDocsWriter.XmlFile ) | Out-NtapDocument -WordFile <WordFile> -ExcelFile <ExcelFile> -CustomerName <CustomerName> -CustomerLocation <CustomerLocation>")
                [void]$msg.AppendLine('--------------------------------------------------------------------------------------------------')
                [void]$msg.AppendLine()

                Write-Error -ErrorRecord (New-Object -TypeName System.Management.Automation.ErrorRecord -ArgumentList ($msg.ToString(), 'WriteError', [System.Management.Automation.ErrorCategory]::WriteError, $ntapDocsWriter.ExcelFile))
            }
        }

        $statsParams = @{
            FunctionPSCallStack  = $functionPSCallStack
            ElapsedTicks         = ([datetime]::Now - $functionStartTime).Ticks
            BaseTypeName         = $dataSet.ExtendedProperties['BaseTypeName']
            DataSetName          = $dataSet.DataSetName
            ParameterToSetAsBool = 'CustomerName', 'CustomerLocation'
        }

        Write-Statistics @statsParams
    }
}

# SIG # Begin signature block
# MIIq0AYJKoZIhvcNAQcCoIIqwTCCKr0CAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCBSb80cGeSG0ixU
# wCruHPVr/WOoveZIweJfROkCwoP7LqCCJGgwggVvMIIEV6ADAgECAhBI/JO0YFWU
# jTanyYqJ1pQWMA0GCSqGSIb3DQEBDAUAMHsxCzAJBgNVBAYTAkdCMRswGQYDVQQI
# DBJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNVBAcMB1NhbGZvcmQxGjAYBgNVBAoM
# EUNvbW9kbyBDQSBMaW1pdGVkMSEwHwYDVQQDDBhBQUEgQ2VydGlmaWNhdGUgU2Vy
# dmljZXMwHhcNMjEwNTI1MDAwMDAwWhcNMjgxMjMxMjM1OTU5WjBWMQswCQYDVQQG
# EwJHQjEYMBYGA1UEChMPU2VjdGlnbyBMaW1pdGVkMS0wKwYDVQQDEyRTZWN0aWdv
# IFB1YmxpYyBDb2RlIFNpZ25pbmcgUm9vdCBSNDYwggIiMA0GCSqGSIb3DQEBAQUA
# A4ICDwAwggIKAoICAQCN55QSIgQkdC7/FiMCkoq2rjaFrEfUI5ErPtx94jGgUW+s
# hJHjUoq14pbe0IdjJImK/+8Skzt9u7aKvb0Ffyeba2XTpQxpsbxJOZrxbW6q5KCD
# J9qaDStQ6Utbs7hkNqR+Sj2pcaths3OzPAsM79szV+W+NDfjlxtd/R8SPYIDdub7
# P2bSlDFp+m2zNKzBenjcklDyZMeqLQSrw2rq4C+np9xu1+j/2iGrQL+57g2extme
# me/G3h+pDHazJyCh1rr9gOcB0u/rgimVcI3/uxXP/tEPNqIuTzKQdEZrRzUTdwUz
# T2MuuC3hv2WnBGsY2HH6zAjybYmZELGt2z4s5KoYsMYHAXVn3m3pY2MeNn9pib6q
# RT5uWl+PoVvLnTCGMOgDs0DGDQ84zWeoU4j6uDBl+m/H5x2xg3RpPqzEaDux5mcz
# mrYI4IAFSEDu9oJkRqj1c7AGlfJsZZ+/VVscnFcax3hGfHCqlBuCF6yH6bbJDoEc
# QNYWFyn8XJwYK+pF9e+91WdPKF4F7pBMeufG9ND8+s0+MkYTIDaKBOq3qgdGnA2T
# OglmmVhcKaO5DKYwODzQRjY1fJy67sPV+Qp2+n4FG0DKkjXp1XrRtX8ArqmQqsV/
# AZwQsRb8zG4Y3G9i/qZQp7h7uJ0VP/4gDHXIIloTlRmQAOka1cKG8eOO7F/05QID
# AQABo4IBEjCCAQ4wHwYDVR0jBBgwFoAUoBEKIz6W8Qfs4q8p74Klf9AwpLQwHQYD
# VR0OBBYEFDLrkpr/NZZILyhAQnAgNpFcF4XmMA4GA1UdDwEB/wQEAwIBhjAPBgNV
# HRMBAf8EBTADAQH/MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBsGA1UdIAQUMBIwBgYE
# VR0gADAIBgZngQwBBAEwQwYDVR0fBDwwOjA4oDagNIYyaHR0cDovL2NybC5jb21v
# ZG9jYS5jb20vQUFBQ2VydGlmaWNhdGVTZXJ2aWNlcy5jcmwwNAYIKwYBBQUHAQEE
# KDAmMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5jb20wDQYJKoZI
# hvcNAQEMBQADggEBABK/oe+LdJqYRLhpRrWrJAoMpIpnuDqBv0WKfVIHqI0fTiGF
# OaNrXi0ghr8QuK55O1PNtPvYRL4G2VxjZ9RAFodEhnIq1jIV9RKDwvnhXRFAZ/ZC
# J3LFI+ICOBpMIOLbAffNRk8monxmwFE2tokCVMf8WPtsAO7+mKYulaEMUykfb9gZ
# pk+e96wJ6l2CxouvgKe9gUhShDHaMuwV5KZMPWw5c9QLhTkg4IUaaOGnSDip0TYl
# d8GNGRbFiExmfS9jzpjoad+sPKhdnckcW67Y8y90z7h+9teDnRGWYpquRRPaf9xH
# +9/DUp/mBlXpnYzyOmJRvOwkDynUWICE5EV7WtgwggWNMIIEdaADAgECAhAOmxiO
# +dAt5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVTMRUwEwYD
# VQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xJDAi
# BgNVBAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4MDEwMDAw
# MDBaFw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERp
# Z2lDZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQADggIPADCC
# AgoCggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVirdprNrnsb
# hA3EMB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcWWVVyr2iT
# cMKyunWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5OyJP4IWGb
# NOsFxl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7Ke13jrclP
# XuU15zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1gj4QkXCr
# VYJBMtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn3aQnvKFP
# ObURWBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7nDmOu5tTv
# kpI6nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIRt7t/8tWM
# cCxBYKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEdslQpJYls
# 5Q5SUUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j7CFfxCBR
# a2+xq4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMBAAGjggE6
# MIIBNjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzfLmc/57qY
# rhwPTzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNVHQ8BAf8E
# BAMCAYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5k
# aWdpY2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4wPDA6oDig
# NoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9v
# dENBLmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQADggEBAHCg
# v0NcVec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3bb0aFPQT
# SnovLbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP0UNEm0Mh
# 65ZyoUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZNUZqaVSw
# uKFWjuyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPLILCsWKAO
# QGPFmCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9W9FcrBjD
# TZ9ztwGpn1eqXijiuZQwggXMMIIENKADAgECAhAg429sPxgagUb53pPffJfkMA0G
# CSqGSIb3DQEBDAUAMFQxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExp
# bWl0ZWQxKzApBgNVBAMTIlNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBDQSBS
# MzYwHhcNMjEwOTA5MDAwMDAwWhcNMjMwOTA5MjM1OTU5WjBQMQswCQYDVQQGEwJV
# UzETMBEGA1UECAwKQ2FsaWZvcm5pYTEVMBMGA1UECgwMTmV0QXBwLCBJbmMuMRUw
# EwYDVQQDDAxOZXRBcHAsIEluYy4wggGiMA0GCSqGSIb3DQEBAQUAA4IBjwAwggGK
# AoIBgQC4kYYj/oViZD9pN03hrqFjtwOz1Gx4eDWVV8IYUYEr2qpLeNYvWz26B/+E
# mYLboAGVpkYg0Wske3hjRpooZlMORm1S4+2C2NoQCstZ+kmNTevbXHJe/w1VNJrm
# fKjpXOJEfx8GUGDqCMR30dChVpsdBEavrM7T0hnfJLv18i19SGv3a+nhvB3iOjLP
# SEg8+DFkjD5jTI6fQ83DBEiVR4UEYq7yyH578ZEylhsRfJmK+uIREycEUk/NpFTw
# g6/7lxh/WYabCrZzqI4Ep3QataRKaAlxt3BiOlZkX4WGX3DYvSYltWOM2UlCcGpP
# m/1/LN8Xuvf+YO6H472QecHe59XtXbHSMKvqDuOEJu8Wk2LFMNK732o3Fc5QIHie
# 6JtNsgNYf+Vsm5EMBD1ZHNE+C3zlmQbbWIU5uEU1nhiodBPKsu0SkS6pARyKBO05
# DSc2gWWv0aukKi04ZAn+hAdSl5s1dgmU5shBvKHqe15K9ZRN4FFO/CgHE0BoGYQS
# UQVKwa0CAwEAAaOCAZwwggGYMB8GA1UdIwQYMBaAFA8qyyCHKLjsb0iuK1SmKaoX
# pM0MMB0GA1UdDgQWBBQuH643KcBMmb/Q6IZt+H9IrnXFwDAOBgNVHQ8BAf8EBAMC
# B4AwDAYDVR0TAQH/BAIwADATBgNVHSUEDDAKBggrBgEFBQcDAzARBglghkgBhvhC
# AQEEBAMCBBAwSgYDVR0gBEMwQTA1BgwrBgEEAbIxAQIBAwIwJTAjBggrBgEFBQcC
# ARYXaHR0cHM6Ly9zZWN0aWdvLmNvbS9DUFMwCAYGZ4EMAQQBMEkGA1UdHwRCMEAw
# PqA8oDqGOGh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0NvZGVT
# aWduaW5nQ0FSMzYuY3JsMHkGCCsGAQUFBwEBBG0wazBEBggrBgEFBQcwAoY4aHR0
# cDovL2NydC5zZWN0aWdvLmNvbS9TZWN0aWdvUHVibGljQ29kZVNpZ25pbmdDQVIz
# Ni5jcnQwIwYIKwYBBQUHMAGGF2h0dHA6Ly9vY3NwLnNlY3RpZ28uY29tMA0GCSqG
# SIb3DQEBDAUAA4IBgQCOoGdXjP8Sif0h3ZvoDsIVfrJvQUdP9pZExRJGGj/Te6ML
# XyqHNc/G2WklNv+BC4ryNJ4zp0oneHOub55ejrYPfhgP9oFWS8oOPJbrp3rAtINa
# OeKRC88WUYZSKYAxSX6kubEjnQD6cmazXjxVN6P8+6q9smeqF3xI+++c+ekMw3Xv
# 4EWRYXqN0/srfFp1WpoluwSQcZMqj9haSX0bgO6AAWH2AnVJMfXTIKz/0FQ/RW0y
# Ed5QYQqYInhA7IUz9dBziNpJlwONeF5j7xzgfYDY63WU6WrgJnGjYkQCOOsUx74j
# gUiMRe9zV48GS8Vxz22c/TQs0x4/1GmOSJvUHEk3GseBmB3v+yEhh/D6zWOfYP4X
# D/9b91CxmugTuepuaJSeRg+qUm3KarKsOCUF/CLqUrNh/JwKrWD1cghRaYLvMucs
# ScksHHe7ZDvb2OtvxWXjPk1d1NKvEwFJSS6hIVIbug9x28AJqOoP5Pn9wZvJSzvW
# uQJGCLscdGyKefdCo30wggYaMIIEAqADAgECAhBiHW0MUgGeO5B5FSCJIRwKMA0G
# CSqGSIb3DQEBDAUAMFYxCzAJBgNVBAYTAkdCMRgwFgYDVQQKEw9TZWN0aWdvIExp
# bWl0ZWQxLTArBgNVBAMTJFNlY3RpZ28gUHVibGljIENvZGUgU2lnbmluZyBSb290
# IFI0NjAeFw0yMTAzMjIwMDAwMDBaFw0zNjAzMjEyMzU5NTlaMFQxCzAJBgNVBAYT
# AkdCMRgwFgYDVQQKEw9TZWN0aWdvIExpbWl0ZWQxKzApBgNVBAMTIlNlY3RpZ28g
# UHVibGljIENvZGUgU2lnbmluZyBDQSBSMzYwggGiMA0GCSqGSIb3DQEBAQUAA4IB
# jwAwggGKAoIBgQCbK51T+jU/jmAGQ2rAz/V/9shTUxjIztNsfvxYB5UXeWUzCxEe
# AEZGbEN4QMgCsJLZUKhWThj/yPqy0iSZhXkZ6Pg2A2NVDgFigOMYzB2OKhdqfWGV
# oYW3haT29PSTahYkwmMv0b/83nbeECbiMXhSOtbam+/36F09fy1tsB8je/RV0mIk
# 8XL/tfCK6cPuYHE215wzrK0h1SWHTxPbPuYkRdkP05ZwmRmTnAO5/arnY83jeNzh
# P06ShdnRqtZlV59+8yv+KIhE5ILMqgOZYAENHNX9SJDm+qxp4VqpB3MV/h53yl41
# aHU5pledi9lCBbH9JeIkNFICiVHNkRmq4TpxtwfvjsUedyz8rNyfQJy/aOs5b4s+
# ac7IH60B+Ja7TVM+EKv1WuTGwcLmoU3FpOFMbmPj8pz44MPZ1f9+YEQIQty/NQd/
# 2yGgW+ufflcZ/ZE9o1M7a5Jnqf2i2/uMSWymR8r2oQBMdlyh2n5HirY4jKnFH/9g
# Rvd+QOfdRrJZb1sCAwEAAaOCAWQwggFgMB8GA1UdIwQYMBaAFDLrkpr/NZZILyhA
# QnAgNpFcF4XmMB0GA1UdDgQWBBQPKssghyi47G9IritUpimqF6TNDDAOBgNVHQ8B
# Af8EBAMCAYYwEgYDVR0TAQH/BAgwBgEB/wIBADATBgNVHSUEDDAKBggrBgEFBQcD
# AzAbBgNVHSAEFDASMAYGBFUdIAAwCAYGZ4EMAQQBMEsGA1UdHwREMEIwQKA+oDyG
# Omh0dHA6Ly9jcmwuc2VjdGlnby5jb20vU2VjdGlnb1B1YmxpY0NvZGVTaWduaW5n
# Um9vdFI0Ni5jcmwwewYIKwYBBQUHAQEEbzBtMEYGCCsGAQUFBzAChjpodHRwOi8v
# Y3J0LnNlY3RpZ28uY29tL1NlY3RpZ29QdWJsaWNDb2RlU2lnbmluZ1Jvb3RSNDYu
# cDdjMCMGCCsGAQUFBzABhhdodHRwOi8vb2NzcC5zZWN0aWdvLmNvbTANBgkqhkiG
# 9w0BAQwFAAOCAgEABv+C4XdjNm57oRUgmxP/BP6YdURhw1aVcdGRP4Wh60BAscjW
# 4HL9hcpkOTz5jUug2oeunbYAowbFC2AKK+cMcXIBD0ZdOaWTsyNyBBsMLHqafvIh
# rCymlaS98+QpoBCyKppP0OcxYEdU0hpsaqBBIZOtBajjcw5+w/KeFvPYfLF/ldYp
# mlG+vd0xqlqd099iChnyIMvY5HexjO2AmtsbpVn0OhNcWbWDRF/3sBp6fWXhz7Dc
# ML4iTAWS+MVXeNLj1lJziVKEoroGs9Mlizg0bUMbOalOhOfCipnx8CaLZeVme5yE
# Lg09Jlo8BMe80jO37PU8ejfkP9/uPak7VLwELKxAMcJszkyeiaerlphwoKx1uHRz
# NyE6bxuSKcutisqmKL5OTunAvtONEoteSiabkPVSZ2z76mKnzAfZxCl/3dq3dUNw
# 4rg3sTCggkHSRqTqlLMS7gjrhTqBmzu1L90Y1KWN/Y5JKdGvspbOrTfOXyXvmPL6
# E52z1NZJ6ctuMFBQZH3pwWvqURR8AgQdULUvrxjUYbHHj95Ejza63zdrEcxWLDX6
# xWls/GDnVNueKjWUH3fTv1Y8Wdho698YADR7TNx8X8z2Bev6SivBBOHY+uqiirZt
# g0y9ShQoPzmCcn63Syatatvx157YK9hlcPmVoa1oDE5/L9Uo2bC5a4CH2Rwwggau
# MIIElqADAgECAhAHNje3JFR82Ees/ShmKl5bMA0GCSqGSIb3DQEBCwUAMGIxCzAJ
# BgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5k
# aWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBHNDAe
# Fw0yMjAzMjMwMDAwMDBaFw0zNzAzMjIyMzU5NTlaMGMxCzAJBgNVBAYTAlVTMRcw
# FQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1c3Rl
# ZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3RhbXBpbmcgQ0EwggIiMA0GCSqGSIb3
# DQEBAQUAA4ICDwAwggIKAoICAQDGhjUGSbPBPXJJUVXHJQPE8pE3qZdRodbSg9Ge
# TKJtoLDMg/la9hGhRBVCX6SI82j6ffOciQt/nR+eDzMfUBMLJnOWbfhXqAJ9/UO0
# hNoR8XOxs+4rgISKIhjf69o9xBd/qxkrPkLcZ47qUT3w1lbU5ygt69OxtXXnHwZl
# jZQp09nsad/ZkIdGAHvbREGJ3HxqV3rwN3mfXazL6IRktFLydkf3YYMZ3V+0VAsh
# aG43IbtArF+y3kp9zvU5EmfvDqVjbOSmxR3NNg1c1eYbqMFkdECnwHLFuk4fsbVY
# TXn+149zk6wsOeKlSNbwsDETqVcplicu9Yemj052FVUmcJgmf6AaRyBD40NjgHt1
# biclkJg6OBGz9vae5jtb7IHeIhTZgirHkr+g3uM+onP65x9abJTyUpURK1h0QCir
# c0PO30qhHGs4xSnzyqqWc0Jon7ZGs506o9UD4L/wojzKQtwYSH8UNM/STKvvmz3+
# DrhkKvp1KCRB7UK/BZxmSVJQ9FHzNklNiyDSLFc1eSuo80VgvCONWPfcYd6T/jnA
# +bIwpUzX6ZhKWD7TA4j+s4/TXkt2ElGTyYwMO1uKIqjBJgj5FBASA31fI7tk42Pg
# puE+9sJ0sj8eCXbsq11GdeJgo1gJASgADoRU7s7pXcheMBK9Rp6103a50g5rmQzS
# M7TNsQIDAQABo4IBXTCCAVkwEgYDVR0TAQH/BAgwBgEB/wIBADAdBgNVHQ4EFgQU
# uhbZbU2FL3MpdpovdYxqII+eyG8wHwYDVR0jBBgwFoAU7NfjgtJxXWRM3y5nP+e6
# mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMIMHcGCCsG
# AQUFBwEBBGswaTAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29t
# MEEGCCsGAQUFBzAChjVodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNl
# cnRUcnVzdGVkUm9vdEc0LmNydDBDBgNVHR8EPDA6MDigNqA0hjJodHRwOi8vY3Js
# My5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNybDAgBgNVHSAE
# GTAXMAgGBmeBDAEEAjALBglghkgBhv1sBwEwDQYJKoZIhvcNAQELBQADggIBAH1Z
# jsCTtm+YqUQiAX5m1tghQuGwGC4QTRPPMFPOvxj7x1Bd4ksp+3CKDaopafxpwc8d
# B+k+YMjYC+VcW9dth/qEICU0MWfNthKWb8RQTGIdDAiCqBa9qVbPFXONASIlzpVp
# P0d3+3J0FNf/q0+KLHqrhc1DX+1gtqpPkWaeLJ7giqzl/Yy8ZCaHbJK9nXzQcAp8
# 76i8dU+6WvepELJd6f8oVInw1YpxdmXazPByoyP6wCeCRK6ZJxurJB4mwbfeKuv2
# nrF5mYGjVoarCkXJ38SNoOeY+/umnXKvxMfBwWpx2cYTgAnEtp/Nh4cku0+jSbl3
# ZpHxcpzpSwJSpzd+k1OsOx0ISQ+UzTl63f8lY5knLD0/a6fxZsNBzU+2QJshIUDQ
# txMkzdwdeDrknq3lNHGS1yZr5Dhzq6YBT70/O3itTK37xJV77QpfMzmHQXh6OOmc
# 4d0j/R0o08f56PGYX/sr2H7yRp11LB4nLCbbbxV7HhmLNriT1ObyF5lZynDwN7+Y
# AN8gFk8n+2BnFqFmut1VwDophrCYoCvtlUG3OtUVmDG0YgkPCr2B2RP+v6TR81fZ
# vAT6gt4y3wSJ8ADNXcL50CN/AAvkdgIm2fBldkKmKYcJRyvmfxqkhQ/8mJb2VVQr
# H4D6wPIOK+XW+6kvRBVK5xMOHds3OBqhK/bt1nz8MIIGwDCCBKigAwIBAgIQDE1p
# ckuU+jwqSj0pB4A9WjANBgkqhkiG9w0BAQsFADBjMQswCQYDVQQGEwJVUzEXMBUG
# A1UEChMORGlnaUNlcnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQg
# RzQgUlNBNDA5NiBTSEEyNTYgVGltZVN0YW1waW5nIENBMB4XDTIyMDkyMTAwMDAw
# MFoXDTMzMTEyMTIzNTk1OVowRjELMAkGA1UEBhMCVVMxETAPBgNVBAoTCERpZ2lD
# ZXJ0MSQwIgYDVQQDExtEaWdpQ2VydCBUaW1lc3RhbXAgMjAyMiAtIDIwggIiMA0G
# CSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDP7KUmOsap8mu7jcENmtuh6BSFdDMa
# JqzQHFUeHjZtvJJVDGH0nQl3PRWWCC9rZKT9BoMW15GSOBwxApb7crGXOlWvM+xh
# iummKNuQY1y9iVPgOi2Mh0KuJqTku3h4uXoW4VbGwLpkU7sqFudQSLuIaQyIxvG+
# 4C99O7HKU41Agx7ny3JJKB5MgB6FVueF7fJhvKo6B332q27lZt3iXPUv7Y3UTZWE
# aOOAy2p50dIQkUYp6z4m8rSMzUy5Zsi7qlA4DeWMlF0ZWr/1e0BubxaompyVR4aF
# eT4MXmaMGgokvpyq0py2909ueMQoP6McD1AGN7oI2TWmtR7aeFgdOej4TJEQln5N
# 4d3CraV++C0bH+wrRhijGfY59/XBT3EuiQMRoku7mL/6T+R7Nu8GRORV/zbq5Xwx
# 5/PCUsTmFntafqUlc9vAapkhLWPlWfVNL5AfJ7fSqxTlOGaHUQhr+1NDOdBk+lbP
# 4PQK5hRtZHi7mP2Uw3Mh8y/CLiDXgazT8QfU4b3ZXUtuMZQpi+ZBpGWUwFjl5S4p
# kKa3YWT62SBsGFFguqaBDwklU/G/O+mrBw5qBzliGcnWhX8T2Y15z2LF7OF7ucxn
# EweawXjtxojIsG4yeccLWYONxu71LHx7jstkifGxxLjnU15fVdJ9GSlZA076XepF
# cxyEftfO4tQ6dwIDAQABo4IBizCCAYcwDgYDVR0PAQH/BAQDAgeAMAwGA1UdEwEB
# /wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwIAYDVR0gBBkwFzAIBgZngQwB
# BAIwCwYJYIZIAYb9bAcBMB8GA1UdIwQYMBaAFLoW2W1NhS9zKXaaL3WMaiCPnshv
# MB0GA1UdDgQWBBRiit7QYfyPMRTtlwvNPSqUFN9SnDBaBgNVHR8EUzBRME+gTaBL
# hklodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkRzRSU0E0
# MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3JsMIGQBggrBgEFBQcBAQSBgzCBgDAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMFgGCCsGAQUFBzAC
# hkxodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkRzRS
# U0E0MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3J0MA0GCSqGSIb3DQEBCwUAA4IC
# AQBVqioa80bzeFc3MPx140/WhSPx/PmVOZsl5vdyipjDd9Rk/BX7NsJJUSx4iGNV
# CUY5APxp1MqbKfujP8DJAJsTHbCYidx48s18hc1Tna9i4mFmoxQqRYdKmEIrUPwb
# tZ4IMAn65C3XCYl5+QnmiM59G7hqopvBU2AJ6KO4ndetHxy47JhB8PYOgPvk/9+d
# EKfrALpfSo8aOlK06r8JSRU1NlmaD1TSsht/fl4JrXZUinRtytIFZyt26/+YsiaV
# OBmIRBTlClmia+ciPkQh0j8cwJvtfEiy2JIMkU88ZpSvXQJT657inuTTH4YBZJwA
# wuladHUNPeF5iL8cAZfJGSOA1zZaX5YWsWMMxkZAO85dNdRZPkOaGK7DycvD+5sT
# X2q1x+DzBcNZ3ydiK95ByVO5/zQQZ/YmMph7/lxClIGUgp2sCovGSxVK05iQRWAz
# gOAj3vgDpPZFR+XOuANCR+hBNnF3rf2i6Jd0Ti7aHh2MWsgemtXC8MYiqE+bvdgc
# mlHEL5r2X6cnl7qWLoVXwGDneFZ/au/ClZpLEQLIgpzJGgV8unG1TnqZbPTontRa
# mMifv427GFxD9dAq6OJi7ngE273R+1sKqHB+8JeEeOMIA11HLGOoJTiXAdI/Otrl
# 5fbmm9x+LMz/F0xNAKLY1gEOuIvu5uByVYksJxlh9ncBjDGCBb4wggW6AgEBMGgw
# VDELMAkGA1UEBhMCR0IxGDAWBgNVBAoTD1NlY3RpZ28gTGltaXRlZDErMCkGA1UE
# AxMiU2VjdGlnbyBQdWJsaWMgQ29kZSBTaWduaW5nIENBIFIzNgIQIONvbD8YGoFG
# +d6T33yX5DANBglghkgBZQMEAgEFAKCBhDAYBgorBgEEAYI3AgEMMQowCKACgACh
# AoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAM
# BgorBgEEAYI3AgEVMC8GCSqGSIb3DQEJBDEiBCAiQk887x3UGySX2NRxQBkC3+BU
# oK9ML2eHfy51GOm0xTANBgkqhkiG9w0BAQEFAASCAYBaVSBCosHHIQ3UCOlDz2JW
# ylYjreWtinPGJpCtnmGTMiO6/vfBoc/Vfbvn7q6w4B2tKv3+tJ8kwrHRgF+fn7bF
# 2ScVj4XiNjkFnfD/TfWtaS5RntCnYQlDNfJ8TZGkPzvtws/ryHRRXwcJLbExoMNM
# g6JyI0/16EzMUTTn80/v1Z3WJ3PmlWopwBxydnzAnrHC1aV4LyaeoRyPRVE/k4MZ
# 61fL30nc/cs7cDru0upsgjUWV1hZOjUpoPbpy5kpTU5cPPFnYb5pJLATRoaxIF7j
# usipfYpWJgJj45scIGZzfdcyqGRZ4Wd4jtJOvhSUyVEN3N+RTsBKrlmSuCqXtkiw
# 5pMjGax+VvN1zFqJykaxg2MjwUzKR0Dw11okwVJngnOtcRVGIGD7cKEPZhtApDSE
# K5uNB/n+CwxPlX08wGCqVHa//VPENTNNJHGoRQZKJ4TRGcNtdUNsVtSibXjEupU4
# PMRCem+dyiXdebPhfDmV3Pw/SyuJHjiEGYRIBp3IX5KhggMgMIIDHAYJKoZIhvcN
# AQkGMYIDDTCCAwkCAQEwdzBjMQswCQYDVQQGEwJVUzEXMBUGA1UEChMORGlnaUNl
# cnQsIEluYy4xOzA5BgNVBAMTMkRpZ2lDZXJ0IFRydXN0ZWQgRzQgUlNBNDA5NiBT
# SEEyNTYgVGltZVN0YW1waW5nIENBAhAMTWlyS5T6PCpKPSkHgD1aMA0GCWCGSAFl
# AwQCAQUAoGkwGAYJKoZIhvcNAQkDMQsGCSqGSIb3DQEHATAcBgkqhkiG9w0BCQUx
# DxcNMjMwNDAyMTUyODI4WjAvBgkqhkiG9w0BCQQxIgQgwZmwq2FrHWO9OSm841WH
# +mNsEGTjWKLAoBkogX2HJpMwDQYJKoZIhvcNAQEBBQAEggIAk9mbYCd+whErK67I
# 5HBLjl5rDwFJUnyGwOraSbV+/t0BRUgmedrT7sP/sIm195NdtgizPWcSUh1xR0JG
# goFsTKKtRN7hB/n2EGOrtMMvnXmw7qElSoBV+aNUQdiUe6oydvl+7LNL7WBl+pLv
# 0R+WXusX7yy3rhedOXc7KEOqy1WLSnwxrxcd2jLDBKXYCLmHxv9yA8Oz8tXztxDE
# Gyb+2PN2ybj4A65TmskF2WynnbO4y/PxvH39hq7AwOl9+4Tc7723NXzCZ7fkXYms
# 2uB72YdfiIEdghIJYuBdCc0nYWp/XM+DX7qVlEqElXcL0q1bqFKy6zd7CznkjWjM
# 1PhXH90up8DBIy2fBh2MhqP5CrRbp3udXzU+JYYDqeul0aWsMCrt1XBeywvuS13h
# i7REAEKcgcGlY6+tj/T/07ngENMx2Ae18BbmDqi8Vf1vIYPjnwRkAIajfjaoyZI7
# gEIzDXz+MYjnmr5j3qYUvRCKcTvmQ0XrT0K8RSz2+e0ZELvM1wd9IGfjIAUrv2j+
# LhDIpJFpPAcq3rtrZWZ49GW+7hmMQzlaOulA056nII50QCOObZZqIpbWGS4uR73O
# 60i0B0lu9Pdrr2Tq6CyxHnhspPIwPtbBx6Xs/M4TXA+sbBb9nivKhqNs694Kzyfw
# kTi763k75N8kf0aqM8Euqzzrdo8=
# SIG # End signature block
